@@ -6,7 +6,7 @@
 ;; Maintainer: Protesilaos Stavrou General Issues <~protesilaos/general-issues@lists.sr.ht>
 ;; URL: https://git.sr.ht/~protesilaos/beframe
 ;; Mailing-List: https://lists.sr.ht/~protesilaos/general-issues
-;; Version: 0.1.8
+;; Version: 0.1.10
 ;; Package-Requires: ((emacs "28.1"))
 
 ;; This file is NOT part of GNU Emacs.
@@ -69,6 +69,14 @@
 ;; frame-specific scratch buffer after the frame is deleted.  Set this
 ;; user option to `nil' to disable the killing of such buffers.
 ;;
+;; The command `beframe-assume-frame-buffers' (alias
+;; `beframe-add-frame-buffers') prompts for a frame and then copies
+;; its buffer list into the current frame.
+;;
+;; The command `beframe-unassume-frame-buffers' (alias
+;; `beframe-remove-frame-buffers') prompts for a frame and then
+;; removes its buffer list from the current frame.
+;;
 ;; The `beframe-mode' does the following:
 ;;
 ;; - Sets the value of `read-buffer-function' to a function that
@@ -362,8 +370,45 @@ its placement and other parameters."
    (list
     (when current-prefix-arg
       (beframe--frame-object (beframe--frame-prompt)))))
-  (display-buffer (beframe--list-buffers-noselect frame))
-  (buffer-menu--display-help))
+  (display-buffer (beframe--list-buffers-noselect frame)))
+
+;;;###autoload
+(defun beframe-assume-frame-buffers (frame)
+  "Assume FRAME buffer list, copying it into current buffer list.
+When called interactively, prompt for FRAME using completion.
+Else FRAME must satisfy `framep'.
+
+Also see `beframe-unassume-frame-buffers'."
+  (interactive (list (beframe--frame-object (beframe--frame-prompt))))
+  (let* ((other-buffer-list (beframe--buffer-list frame))
+         (buffers (delete-dups (append other-buffer-list (beframe--buffer-list)))))
+    (modify-frame-parameters
+     nil
+     `((buffer-list . ,buffers)))))
+
+(defalias 'beframe-add-frame-buffers 'beframe-assume-frame-buffers
+  "Alias of `beframe-assume-frame-buffers' command.")
+
+;;;###autoload
+(defun beframe-unassume-frame-buffers (frame)
+  "Unassume FRAME buffer list, removing it from current buffer list.
+When called interactively, prompt for FRAME using completion.
+Else FRAME must satisfy `framep'.
+
+Also see `beframe-assume-frame-buffers'."
+  (interactive (list (beframe--frame-object (beframe--frame-prompt))))
+  (let* ((other-buffer-list (beframe--buffer-list frame))
+         (new-buffer-list
+          (seq-filter
+           (lambda (buf)
+             (not (member buf other-buffer-list)))
+           (beframe--buffer-list))))
+    (modify-frame-parameters
+     nil
+     `((buffer-list . ,new-buffer-list)))))
+
+(defalias 'beframe-remove-frame-buffers 'beframe-unassume-frame-buffers
+  "Alias of `beframe-unassume-frame-buffers' command.")
 
 ;;; Minor mode setup
 
@@ -379,7 +424,8 @@ its placement and other parameters."
         (setq beframe--read-buffer-function read-buffer-function
               read-buffer-function #'beframe-read-buffer)
         (add-hook 'after-make-frame-functions #'beframe--frame-predicate)
-        (add-hook 'after-make-frame-functions beframe-rename-function)
+        (when beframe-rename-function
+          (add-hook 'after-make-frame-functions beframe-rename-function))
         (add-hook 'after-make-frame-functions #'beframe-create-scratch-buffer)
         (beframe--functions-in-frames))
     (setq read-buffer-function beframe--read-buffer-function
@@ -395,7 +441,8 @@ its placement and other parameters."
          (buf (get-buffer-create (format "*scratch for %s*" name))))
     (with-current-buffer buf
       (funcall initial-major-mode)
-      (when (zerop (buffer-size))
+      (when (and (zerop (buffer-size))
+                 (stringp initial-scratch-message))
         (insert initial-scratch-message))
       (add-hook 'delete-frame-functions
                 (lambda (frame)
@@ -403,22 +450,77 @@ its placement and other parameters."
                              (null frame))
                     (kill-buffer buf)))))
     (let* ((frame-bufs (beframe--buffer-list frame))
-           (frame-bufs-with-buf (push buf frame-bufs)))
+           (frame-bufs-with-buf (append frame-bufs (list buf))))
       (modify-frame-parameters
        frame
        `((buffer-list . ,frame-bufs-with-buf))))))
 
-(defun beframe-rename-frame (frame)
-  "Rename FRAME per `beframe-rename-function'."
-  (select-frame frame)
-  (set-frame-name
-   (cond
-    ((and (not (minibufferp)) (buffer-file-name))
-     (format "%s  %s" (buffer-name) default-directory))
-    ((not (minibufferp))
-     (buffer-name))
-    (t
-     default-directory))))
+(defvar beframe--rename-frame-history nil
+  "Minibuffer history for `beframe-rename-frame'.")
+
+;; (defun beframe--rename-scratch-buffer (frame name)
+;;   "Rename the scratch buffer associated with FRAME according to NAME."
+;;   (when-let* ((buf (get-buffer (format "*scratch for %s*" frame)))
+;;               ((member (format "*scratch for %s*" frame) (beframe--buffer-list))))
+;;     (with-current-buffer buf
+;;       (rename-buffer (format "*scratch for %s*" name)))))
+
+(defun beframe--infer-frame-name (frame name)
+  "Infer a suitable name for FRAME with given NAME.
+See `beframe-rename-frame'."
+  (let* ((buffer (car (frame-parameter frame 'buffer-list)))
+         (file-name (when (bufferp buffer)
+                      (buffer-file-name buffer)))
+         (buf-name (buffer-name buffer))
+         (dir (with-current-buffer buffer
+                default-directory)))
+    (cond
+     ((and name (stringp name))
+      name)
+     ((and (not (minibufferp)) file-name)
+      (format "%s  %s" buf-name dir))
+     ((not (minibufferp))
+      buf-name)
+     (t
+      dir))))
+
+;;;###autoload
+(defun beframe-rename-frame (frame &optional name)
+  "Rename FRAME per `beframe-rename-function'.
+
+When called interactively, prompt for FRAME.  Else accept FRAME
+if it is an object that satisfies `framep'.
+
+With optional NAME as a string, use it to name the given FRAME.
+When called interactively, prompt for NAME when a prefix argument
+is given.
+
+With no NAME argument try to infer a name based on the following:
+
+- If the current window has a buffer that visits a file, name the
+  FRAME after the file's name and its `default-directory'.
+
+- If the current window has a non-file-visiting buffer, use the
+  `buffer-name' as the FRAME name.
+
+- Else use the `default-directory'.
+
+Remember that this function doubles as an example for
+`beframe-rename-function': copy it and modify it accordingly."
+  (interactive
+   (let ((select-frame (beframe--frame-prompt)))
+     (list
+      (beframe--frame-object select-frame)
+      (when current-prefix-arg
+        (read-string
+         (format "Rename the frame now called `%s' to: "
+                 select-frame)
+         nil 'beframe--rename-frame-history select-frame)))))
+  ;; (when name
+  ;;   (beframe--rename-scratch-buffer frame name))
+  (modify-frame-parameters
+   frame
+   (list (cons 'name (beframe--infer-frame-name frame name)))))
 
 (defun beframe--frame-buffer-p (buf &optional frame)
   "Return non-nil if BUF belongs to the current frame.
